@@ -57,7 +57,7 @@ def set_random_seed(seed):
 class DensityFlow(nn.Module):
     def __init__(self,
                  input_size: int,
-                 codebook_size: int = 100,
+                 codebook_size: int = 30,
                  cell_factor_size: int = 0,
                  turn_off_cell_specific: bool = False,
                  supervised_mode: bool = False,
@@ -113,6 +113,7 @@ class DensityFlow(nn.Module):
         self.setup_networks()
         
         print(f"🧬 DensityFlow Initialized:")
+        print(f"   - Codebook size: {self.code_size}")
         print(f"   - Latent Dimension: {self.latent_dim}")
         print(f"   - Gene Dimension: {self.input_size}")
         print(f"   - Hidden Dimensions: {self.hidden_layers}")
@@ -397,7 +398,7 @@ class DensityFlow(nn.Module):
                     rate = theta * torch.sum(xs, dim=1, keepdim=True)
 
             if self.loss_func == 'negbinomial':
-                logits = (mu.log()-dispersion.log()).clamp(min=-15, max=15)
+                logits = (mu.log()-dispersion.log()).clamp(min=-10, max=10)
                 if self.use_zeroinflate:
                     pyro.sample('x', dist.ZeroInflatedDistribution(dist.NegativeBinomial(total_count=dispersion, 
                                                                                          logits=logits),gate_logits=gate_logits).to_event(1), obs=xs)
@@ -486,7 +487,7 @@ class DensityFlow(nn.Module):
                     rate = theta * torch.sum(xs, dim=1, keepdim=True)
 
             if self.loss_func == 'negbinomial':
-                logits = (mu.log()-dispersion.log()).clamp(min=-15, max=15)
+                logits = (mu.log()-dispersion.log()).clamp(min=-10, max=10)
                 if self.use_zeroinflate:
                     pyro.sample('x', dist.ZeroInflatedDistribution(dist.NegativeBinomial(total_count=dispersion, logits=logits),gate_logits=gate_logits).to_event(1), obs=xs)
                 else:
@@ -585,7 +586,7 @@ class DensityFlow(nn.Module):
                     rate = theta * torch.sum(xs, dim=1, keepdim=True)
 
             if self.loss_func == 'negbinomial':
-                logits = (mu.log()-dispersion.log()).clamp(min=-15, max=15)
+                logits = (mu.log()-dispersion.log()).clamp(min=-10, max=10)
                 if self.use_zeroinflate:
                     pyro.sample('x', dist.ZeroInflatedDistribution(dist.NegativeBinomial(total_count=dispersion, 
                                                                                          logits=logits),gate_logits=gate_logits).to_event(1), obs=xs)
@@ -695,7 +696,7 @@ class DensityFlow(nn.Module):
                     rate = theta * torch.sum(xs, dim=1, keepdim=True)
 
             if self.loss_func == 'negbinomial':
-                logits = (mu.log()-dispersion.log()).clamp(min=-15, max=15)
+                logits = (mu.log()-dispersion.log()).clamp(min=-10, max=10)
                 if self.use_zeroinflate:
                     pyro.sample('x', dist.ZeroInflatedDistribution(dist.NegativeBinomial(total_count=dispersion, 
                                                                                          logits=logits),gate_logits=gate_logits).to_event(1), obs=xs)
@@ -1030,7 +1031,7 @@ class DensityFlow(nn.Module):
             ad = sc.AnnData(xs)
             binarize(ad, threshold=threshold)
             xs = ad.X.copy()
-        else:
+        elif self.loss_func == 'poisson':
             xs = np.round(xs)
             
         if sparse.issparse(xs):
@@ -1193,6 +1194,7 @@ class DensityFlow(nn.Module):
                 model = pickle.load(pickle_file)
                 
         print(f"🧬 DensityFlow Initialized:")
+        print(f"   - Codebook size: {model.code_size}")
         print(f"   - Latent Dimension: {model.latent_dim}")
         print(f"   - Gene Dimension: {model.input_size}")
         print(f"   - Hidden Dimensions: {model.hidden_layers}")
